@@ -4,7 +4,7 @@ $(function() {
 	var editorId = "editor";
 	
 	// Cache dialog jQuery references for later use
-	var $perl_critic_dialog = $('#perl-critic-dialog');
+	var $perlCriticDialog = $('#perl-critic-dialog');
 	var $about_dialog = $('#about-dialog');
 	var $help_dialog = $("#help-dialog");
 	var $runDialog = $("#run-dialog");
@@ -157,9 +157,13 @@ $(function() {
 	editor.on("change", function(cm) {
 			clearInterval(timeoutId);
 			timeoutId = setTimeout(function() {
-				podChanged(cm);
-				perl_critic();
-				pod_check();
+				var mode = cm.getOption("mode");
+				if(mode == "perl") {
+					podChanged(cm);
+					podCheck();
+				} else {
+					$("#pod-tab").html('Not supported for mode "' + mode + '"');
+				}
 			}, 250);
 	});
 
@@ -177,14 +181,17 @@ $(function() {
 		// Trigger theme selection
 		$("#theme_selector").change();
 
-		// Update POD tab at startup
-		podChanged(editor);
+		var mode = editor.getOption("mode");
+		if(mode == "perl") {
+			// Update POD tab at startup
+			podChanged(editor);
 
-		// Run Perl::Critic at startup
-		perl_critic();
+			// Run Pod::Checker at startup
+			podCheck();
+		} else {
+			$("#pod-tab").html('Not supported for mode "' + mode + '"');
+		}
 
-		// Run Pod::Checker at startup
-		pod_check();
 	}, 0);
 
 	// Highlight active line
@@ -192,33 +199,23 @@ $(function() {
 
 	$(".results").hide();
 
-	$("#action-name", $actionsDialog).typeahead({
-		source : function(query, process) {
-			$.ajax({
-				type: 'POST',
-				url: '/actions-typeahead',
-				async: false,
-				data: {'action-name': query},
-				success: function(matches) {
-					process(matches);
-				},
-				dataType: 'json',
-			});
-		}
-	}).change(function() {
-		console.error($(this).val());
-	});;
-	
 	$("#ok-button", $actionsDialog).click(function() {
-		$actionsDialog.modal('hide');
-		var actionName = $("#action-name", $actionsDialog).val();
-		console.error(actionName);
+		
+		var $selectedAction = $("#matched-actions :selected", $actionsDialog);
+		if($selectedAction.size() > 0) {
+			var actionId = $selectedAction.attr('id');
+			$actionsDialog.modal('hide');
+			$(document).trigger(actionId);
+		} else {
+			alert("No action is selected");
+		}
 	});
 
 	$(".actions-button").click(function() {
 		$actionsDialog.modal('show');
 		$("#action").val('');
 		$("#action").focus();
+		findAction();
 	});
 	
 	var findAction = function() {
@@ -233,15 +230,16 @@ $(function() {
 				var html = '';
 				for(var i = 0; i < matches.length; i++) {
 					var a = matches[i];
-					html += '<li id="' + a.id + '"><a href="#">' + a.name +'</a></li>';
+					html += "<option id='" + a.id + "' "  + 
+						((i == 0) ? "selected" : "") + 
+						">" + 
+						a.name + 
+						"</option>";
+				}
+				if(matches.length > 0) {
+					$("#ok-button", $actionsDialog).removeAttr("disabled");
 				}
-				$("#ok-button", $actionsDialog).removeAttr("disabled");
 				$("#matched-actions", $actionsDialog).html(html);
-				$("#matched-actions li", $actionsDialog).click(function() {
-					$actionsDialog.modal('hide');
-					var actionId = $(this).attr('id');
-					$(document).trigger(actionId);
-				});
 			},
 			error:   function(jqXHR, textStatus, errorThrown) {
 				console.error("Error:\n" + textStatus + "\n" + errorThrown);
@@ -266,18 +264,69 @@ $(function() {
 			}
 		}
 	});
+	
+	var findFile = function() {
+		var filename = $("#file", $openFileDialog).val();
+		$("#matched-files", $openFileDialog).empty();
+		$("#ok-button", $openFileDialog).attr("disabled","disabled");
+		$.ajax({
+			type:    "POST",
+			url:     "/find-file",
+			data:    { "filename": filename },
+			success: function(results) {
+				var html = '';
+				for(var i = 0; i < results.length; i++) {
+					var result = results[i];
+					html += "<option id='" + result.id + "' "  + 
+						((i == 0) ? "selected" : "") + 
+						">" + 
+						result.name + 
+						"</option>";
+				}
+				$("#ok-button", $openFileDialog).removeAttr("disabled");
+				$("#matched-files", $openFileDialog).html(html);
+			},
+			error:   function(jqXHR, textStatus, errorThrown) {
+				console.error("Error:\n" + textStatus + "\n" + errorThrown);
+			}
+		});
+	}
+
+	var findFileTimeoutId;
+	$("#file", $openFileDialog).on('input', function() {
+		clearTimeout(findFileTimeoutId);
+		findFileTimeoutId = setTimeout(findFile, 500);
+	});
+
+	$("#file", $openFileDialog).keyup(function(evt) {
+		var keyCode = evt.keyCode;
+		if(keyCode == 40) {
+			$("#matched-files", $openFileDialog).focus();
+		} else if(keyCode == 13) {
+			var $okButton = $("#ok-button", $openFileDialog);
+			if(!$okButton.attr('disabled')) {
+				$okButton.click();
+			}
+		}
+	});
 
 	$("#ok-button", $openFileDialog).click(function() {
-		var filename = $("#search-results option:selected", $openFileDialog).attr('id');
+		var filename = $("#matched-files option:selected", $openFileDialog).attr('id');
 		if (!filename) {
 			console.warn("file name is empty");
 			return;
 		}
 		$.ajax({
-			url:     "/open_file",
+			type:    "POST",
+			url:     "/open-file",
 			data:    { "filename": filename },
-			success: function(code) {
-				editor.setValue(code);
+			success: function(result) {
+				if(result.ok) {
+					editor.setValue(result.value);
+					changeMode(editor, result.mode);
+				} else {
+					alert(result.value);
+				}
 			},
 			error:   function(jqXHR, textStatus, errorThrown) {
 				console.error("Error!" + textStatus + ", " + errorThrown);
@@ -291,7 +340,8 @@ $(function() {
 	});
 
 	$(document).on('action-perl-tidy', function() {
-		$.post('/perl_tidy', {"source": editor.getValue()}, function(data) {
+		var $output = $('#output');
+		$.post('/perl-tidy', {"source": editor.getValue()}, function(data) {
 			if(data.error == '') {
 				editor.setValue(data.source);
 			} else {
@@ -303,8 +353,9 @@ $(function() {
 	$(document).on('action-open-file', function() {
 		$openFileDialog.modal('show');
 		$("#file", $openFileDialog).val('').focus();
-		$("#search-results", $openFileDialog).empty();
+		$("#matched-files", $openFileDialog).empty();
 		$("#ok-button", $openFileDialog).attr("disabled","disabled");
+		findFile();
 	});
 
 	$(document).on('action-open-url', function() {
@@ -314,7 +365,7 @@ $(function() {
 			alert("No URL was provided! Aborting");
 			return;
 		}
-		$.post('/open_url',
+		$.post('/open-url',
 			{ "url": url },
 			function(code) {
 				editor.setValue(code);
@@ -327,7 +378,7 @@ $(function() {
 	});
 
 	$(document).on('action-perl-critic', function() {
-		alert("TODO perl critic");
+		perlCritic();
 	});
 
 	$(document).on('action-options', function() {
@@ -450,9 +501,9 @@ $(function() {
 
 	var perlCriticWidgets = [];
 
-	var perl_critic = function() {
+	var perlCritic = function() {
 
-		$.post('/perl_critic', {"source": editor.getValue(), "severity": $(':selected', '#critic_severity_selector').val()}, function(violations) {
+		$.post('/perl-critic', {"source": editor.getValue(), "severity": $(':selected', '#critic_severity_selector').val()}, function(violations) {
 			for (var i = 0; i < perlCriticWidgets.length; i++) {
 				editor.removeLineWidget(perlCriticWidgets[i].widget);
 			}
@@ -495,24 +546,30 @@ $(function() {
 					html += '<br/><strong>Policy:</strong><br/><a target="_blank" href="https://metacpan.org/module/' + policy + '">' + policy + '</a>';
 					html += '<br/><strong>Diagnositcs:</strong><br/>' + violation.diagnostics.replace(/\n/g, '<br/>');
 
-					$('#myModalLabel', $perl_critic_dialog).html(
+					$('#myModalLabel', $perlCriticDialog).html(
 						violation.description
 					);
-					$('.modal-body', $perl_critic_dialog).html(html);
-					$perl_critic_dialog.modal("show");
+					$('.modal-body', $perlCriticDialog).html(html);
+					$perlCriticDialog.modal("show");
 				});
 			}
+			
+			if(violations.length > 0) {
+				alert("Perl Critic: Found " + violations.length + " violation(s)");
+			} else {
+				alert("Perl Critic: No violations found for current severity level");
+			}
 		});
 	};
 
 	$("#critic_severity_selector").change(function() {
-		perl_critic();
+		perlCritic();
 	});
 
 	var perlCheckerWidgets = [];
 
-	var pod_check = function() {
-		$.post('/pod_check', {"source": editor.getValue()}, function(problems) {
+	var podCheck = function() {
+		$.post('/pod-check', {"source": editor.getValue()}, function(problems) {
 			for (var i = 0; i < perlCheckerWidgets.length; i++) {
 				editor.removeLineWidget(perlCheckerWidgets[i].widget);
 			}
